@@ -6,6 +6,7 @@ use Pod::Usage;
 use Text::Amuse::Preprocessor::HTML qw/html_file_to_muse html_to_muse/;
 use LWP::UserAgent;
 use Getopt::Long;
+use Encode qw/decode_utf8/;
 use utf8;
 binmode STDOUT, ":encoding(utf-8)";
 binmode STDERR, ":encoding(utf-8)";
@@ -16,16 +17,29 @@ html-to-muse.pl
 
 =head1 SYNOPSIS
 
-  html-to-muse.pl [ --encoding utf-8 ] file.html
+  html-to-muse.pl [ options... ] file.html
 
 or
 
   html-to-muse.pl http://example.com/my-file.html
 
-Use --encoding C<encoding-name> to set the encoding of the html file
-processed.
+=head2 Options
 
-If a remote, the switch is ignored.
+=over 4
+
+=item --encoding <encoding-name>
+
+Default to utf-8. The switch is used only on local files.
+
+=item --lang
+
+Set the language of the document in the muse output
+
+=item --title
+
+Set the title of the document (otherwise the filename is used).
+
+=back
 
 The result is printed on the standard output, thus you can use it this way:
 
@@ -42,8 +56,12 @@ L<Text::Amuse::Preprocessor>
 
 my $encoding = 'utf-8';
 my $help;
+my $lang = 'en';
+my $title = '';
 
-GetOptions (encoding => \$encoding,
+GetOptions ("encoding=s" => \$encoding,
+            "lang=s" => \$lang,
+            "title=s" => \$title,
             help => \$help) or die;
 
 if ($help || !@ARGV) {
@@ -52,6 +70,14 @@ if ($help || !@ARGV) {
 }
 
 foreach my $f (@ARGV) {
+    if ($title) {
+        $title = decode_utf8($title);
+    }
+    else {
+        $title = $f;
+    }
+    print "#title $title\n";
+    print "#lang $lang\n\n";
     process_target($f);
 }
 
